--[[*******************************
	* Initilise all the variables *
	*****************************]]
-----------------------------------------
-- Get the mod info from the toc file. --
-----------------------------------------
local modName = select(1, ...)
local modVersion = GetAddOnMetadata(modName, "Version")
local modTitle = GetAddOnMetadata(modName, "Title")
local modDescription = GetAddOnMetadata(modName, "Notes")

-------------------------------
-- create all the stub items --
-------------------------------
local addon = LibStub("AceAddon-3.0"):NewAddon(modName, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")
local L	= LibStub("AceLocale-3.0"):GetLocale(modName)

-----------------------------------------------
-- create the default settings for each user --
-----------------------------------------------
local defaults = {
	char = {
		timeInterval = 0.5,
		active = true,
		big = false,
		glow = false,
	}
}

--------------------------------
-- define the local variables --
--------------------------------
local trackingSomething = true
local visibleToggleFlag = false
local holdFlashFlag = false

local IconOffCache
local IconOnCache
local IconOffPath
local IconOnPath

------------------------------------------------
-- called function every time the timer fires --
------------------------------------------------
function addon:OnUpdate()
	if(not addon.db.char.active) then return end	-- not active so dont worry about doing anything
	if (not trackingSomething or UnitOnTaxi("player") or UnitAffectingCombat("player") or IsOutOfBounds() or IsResting()) then
		if (not holdFlashFlag) then
			addon:ShowBlips(true)	-- turn on blips
			holdFlashFlag = true	-- turn on hold flashing flag
		end
	else
		if (holdFlashFlag) then holdFlashFlag = false end	-- turn off hold flashing flag
		visibleToggleFlag = not visibleToggleFlag	-- toggle visible flag
		addon:ShowBlips(visibleToggleFlag)
	end
end

--------------------------------------------
-- change texture depending on visibility --
--------------------------------------------
function addon:ShowBlips(visible)
	if (visible) then
		Minimap:SetBlipTexture(IconOnPath)
	else
		Minimap:SetBlipTexture(IconOffPath)
	end
end


-----------------------------------------------------------------
-- function to check to see if there is anything being tracked --
-- (this is only called when the user changes a setting)       --
-----------------------------------------------------------------
function addon:MINIMAP_UPDATE_TRACKING()
	trackingSomething = false
	for idx = 1, GetNumTrackingTypes() do
		local name, texture, active, category = GetTrackingInfo(idx)
		if ((category == "spell") and active) then
			trackingSomething = true
		end
	end
end

-------------------------------------------
-- function to change the timer settings --
-------------------------------------------
function addon:ResetTimer()
	addon:CancelAllTimers()
	addon:ScheduleRepeatingTimer("OnUpdate", addon.db.char.timeInterval)
end	

---------------------------------
-- init all the addon settings --
---------------------------------
function addon:OnInitialize()
	addon.db = LibStub("AceDB-3.0"):New("NodeFlashDB", defaults)
	addon:RegisterEvent("MINIMAP_UPDATE_TRACKING")
	addon:RegisterChatCommand("nfl", "ChatCommand")
	addon:RegisterChatCommand("nodeflash", "ChatCommand")
	addon:ResetTimer()
	addon:Print(L["Usage"])
	addon:MINIMAP_UPDATE_TRACKING()
	
	-- Set up cache holders for the node images
	IconOffCache = MinimapCluster:CreateTexture(nil, "BACKGROUND")
	IconOffCache:SetAllPoints()
	IconOffCache:Hide()
	IconOnCache = MinimapCluster:CreateTexture(nil, "BACKGROUND")
	IconOnCache:SetAllPoints()
	IconOnCache:Hide()
	
	-- Set the paths to the node images
	addon:ChangePaths()
end

function addon:OnEnable()
end
function addon:OnDisable()
end

--------------------------------
-- Change paths for the icons --
--------------------------------
function addon:ChangePaths()
	if (addon.db.char.big) then 
		IconOnPath = "Interface\\Addons\\NodeFlash\\Textures\\ObjectIconsBig"
	else
		--IconOnPath = "Interface\\Addons\\NodeFlash\\Textures\\ObjectIconsSmall"
		IconOnPath = "Interface\\MINIMAP\\ObjectIcons"
	end	
	
	IconOffPath = "Interface\\Addons\\NodeFlash\\Textures\\ObjectIconsOff"
	
	IconOffCache:SetTexture(IconOffPath)
	IconOnCache:SetTexture(IconOnPath)
end

---------------------------
-- slash command routine --
---------------------------
function addon:ChatCommand(msg)
	local passedNumber = tonumber(msg)
	msg = string.lower(msg)
	
	if (msg == L["on"]) then
		addon.db.char.active = true
		elepsedTime = 0
		addon:Print(L["Turned flashing on"])
	elseif (msg == L["off"]) then
		addon.db.char.active = false
		addon:ShowBlips(true)
		addon:Print(L["Turned flashing off"])
	elseif (msg == L["big"]) then
		addon.db.char.big = true
		addon:ChangePaths()
		addon:ShowBlips(true)
		addon:Print(L["Nodes Big Size"])
	elseif ((msg == L["small"]) or (msg == L["normal"])) then
		addon.db.char.big = false
		addon:ChangePaths()
		addon:ShowBlips(true)
		addon:Print(L["Nodes Normal Size"])
	elseif (passedNumber and (passedNumber > 50)) then
		addon.db.char.timeInterval = passedNumber / 1000
		addon:CancelAllTimers()
		addon:ResetTimer()
		addon:Print(string.format(L["Flash interval change %s"], addon.db.char.timeInterval))
	elseif (msg == L["reset"]) then
		addon.db:ResetDB()
		trackingSomething = true
		visibleToggleFlag = false
		holdFlashFlag = false
		addon:ResetTimer()
		addon:MINIMAP_UPDATE_TRACKING()
		addon:Print(L["Reset message"])

	else
		addon:Print(L["Usage"])
	end
end

